@@ -1,36 +1,34 @@
 // Allow `&Matrix` in function signatures.
 #![allow(clippy::ptr_arg)]
 
-use ff::{Field, ScalarEngine};
+use ff::PrimeField;
 
 use crate::matrix;
 use crate::matrix::{
     apply_matrix, invert, is_identity, is_invertible, is_square, mat_mul, minor, transpose, Matrix,
-    Scalar,
 };
-use crate::scalar_from_u64;
 
 #[derive(Clone, Debug, PartialEq)]
-pub struct MdsMatrices<E: ScalarEngine> {
-    pub m: Matrix<Scalar<E>>,
-    pub m_inv: Matrix<Scalar<E>>,
-    pub m_hat: Matrix<Scalar<E>>,
-    pub m_hat_inv: Matrix<Scalar<E>>,
-    pub m_prime: Matrix<Scalar<E>>,
-    pub m_double_prime: Matrix<Scalar<E>>,
+pub struct MdsMatrices<F: PrimeField> {
+    pub m: Matrix<F>,
+    pub m_inv: Matrix<F>,
+    pub m_hat: Matrix<F>,
+    pub m_hat_inv: Matrix<F>,
+    pub m_prime: Matrix<F>,
+    pub m_double_prime: Matrix<F>,
 }
 
-pub fn create_mds_matrices<E: ScalarEngine>(t: usize) -> MdsMatrices<E> {
-    let m = generate_mds::<E>(t);
+pub fn create_mds_matrices<F: PrimeField>(t: usize) -> MdsMatrices<F> {
+    let m = generate_mds(t);
     derive_mds_matrices(m)
 }
 
-pub fn derive_mds_matrices<E: ScalarEngine>(m: Matrix<Scalar<E>>) -> MdsMatrices<E> {
-    let m_inv = invert::<E>(&m).unwrap(); // m is MDS so invertible.
-    let m_hat = minor::<E>(&m, 0, 0);
-    let m_hat_inv = invert::<E>(&m_hat).unwrap(); // If this returns None, then `mds_matrix` was not correctly generated.
-    let m_prime = make_prime::<E>(&m);
-    let m_double_prime = make_double_prime::<E>(&m, &m_hat_inv);
+pub fn derive_mds_matrices<F: PrimeField>(m: Matrix<F>) -> MdsMatrices<F> {
+    let m_inv = invert(&m).unwrap(); // m is MDS so invertible.
+    let m_hat = minor(&m, 0, 0);
+    let m_hat_inv = invert(&m_hat).unwrap(); // If this returns None, then `mds_matrix` was not correctly generated.
+    let m_prime = make_prime(&m);
+    let m_double_prime = make_double_prime(&m, &m_hat_inv);
 
     MdsMatrices {
         m,
@@ -47,15 +45,15 @@ pub fn derive_mds_matrices<E: ScalarEngine>(m: Matrix<Scalar<E>>) -> MdsMatrices
 /// (minor to the element in both the row and column) is the identity.
 /// We will pluralize this compact structure `sparse_matrixes` to distinguish from `sparse_matrices` from which they are created.
 #[derive(Debug, Clone, PartialEq)]
-pub struct SparseMatrix<E: ScalarEngine> {
+pub struct SparseMatrix<F: PrimeField> {
     /// `w_hat` is the first column of the M'' matrix. It will be directly multiplied (scalar product) with a row of state elements.
-    pub w_hat: Vec<Scalar<E>>,
+    pub w_hat: Vec<F>,
     /// `v_rest` contains all but the first (already included in `w_hat`).
-    pub v_rest: Vec<Scalar<E>>,
+    pub v_rest: Vec<F>,
 }
 
-impl<E: ScalarEngine> SparseMatrix<E> {
-    pub fn new(m_double_prime: Matrix<Scalar<E>>) -> Self {
+impl<F: PrimeField> SparseMatrix<F> {
+    pub fn new(m_double_prime: Matrix<F>) -> Self {
         assert!(Self::is_sparse_matrix(&m_double_prime));
         let size = matrix::rows(&m_double_prime);
 
@@ -65,16 +63,16 @@ impl<E: ScalarEngine> SparseMatrix<E> {
         Self { w_hat, v_rest }
     }
 
-    pub fn is_sparse_matrix(m: &Matrix<Scalar<E>>) -> bool {
-        is_square(&m) && is_identity::<E>(&minor::<E>(&m, 0, 0))
+    pub fn is_sparse_matrix(m: &Matrix<F>) -> bool {
+        is_square(&m) && is_identity(&minor(&m, 0, 0))
     }
 
     pub fn size(&self) -> usize {
         self.w_hat.len()
     }
 
-    pub fn to_matrix(&self) -> Matrix<Scalar<E>> {
-        let mut m = matrix::make_identity::<E>(self.size());
+    pub fn to_matrix(&self) -> Matrix<F> {
+        let mut m = matrix::make_identity(self.size());
         for (j, elt) in self.w_hat.iter().enumerate() {
             m[j][0] = *elt;
         }
@@ -91,35 +89,35 @@ impl<E: ScalarEngine> SparseMatrix<E> {
 //   - M'' is sparse and replaces M for the round.
 //   - The previous layer's M is then replaced by M x M' = M*.
 //   - M* is likewise factored into M*' and M*'', and the process continues.
-pub fn factor_to_sparse_matrixes<E: ScalarEngine>(
-    base_matrix: Matrix<Scalar<E>>,
+pub fn factor_to_sparse_matrixes<F: PrimeField>(
+    base_matrix: Matrix<F>,
     n: usize,
-) -> (Matrix<Scalar<E>>, Vec<SparseMatrix<E>>) {
-    let (pre_sparse, sparse_matrices) = factor_to_sparse_matrices::<E>(base_matrix, n);
+) -> (Matrix<F>, Vec<SparseMatrix<F>>) {
+    let (pre_sparse, sparse_matrices) = factor_to_sparse_matrices(base_matrix, n);
     let sparse_matrixes = sparse_matrices
         .iter()
-        .map(|m| SparseMatrix::<E>::new(m.to_vec()))
+        .map(|m| SparseMatrix::<F>::new(m.to_vec()))
         .collect::<Vec<_>>();
 
     (pre_sparse, sparse_matrixes)
 }
 
-pub fn factor_to_sparse_matrices<E: ScalarEngine>(
-    base_matrix: Matrix<Scalar<E>>,
+pub fn factor_to_sparse_matrices<F: PrimeField>(
+    base_matrix: Matrix<F>,
     n: usize,
-) -> (Matrix<Scalar<E>>, Vec<Matrix<Scalar<E>>>) {
+) -> (Matrix<F>, Vec<Matrix<F>>) {
     let (pre_sparse, mut all) =
         (0..n).fold((base_matrix.clone(), Vec::new()), |(curr, mut acc), _| {
-            let derived = derive_mds_matrices::<E>(curr);
+            let derived = derive_mds_matrices(curr);
             acc.push(derived.m_double_prime);
-            let new = mat_mul::<E>(&base_matrix, &derived.m_prime).unwrap();
+            let new = mat_mul(&base_matrix, &derived.m_prime).unwrap();
             (new, acc)
         });
     all.reverse();
     (pre_sparse, all)
 }
 
-fn generate_mds<E: ScalarEngine>(t: usize) -> Matrix<Scalar<E>> {
+fn generate_mds<F: PrimeField>(t: usize) -> Matrix<F> {
     // Source: https://github.com/dusk-network/dusk-poseidon-merkle/commit/776c37734ea2e71bb608ce4bc58fdb5f208112a7#diff-2eee9b20fb23edcc0bf84b14167cbfdc
     // Generate x and y values deterministically for the cauchy matrix
     // where x[i] != y[i] to allow the values to be inverted
@@ -128,8 +126,8 @@ fn generate_mds<E: ScalarEngine>(t: usize) -> Matrix<Scalar<E>> {
     // [c d]
     // det(M) = (ad - bc) ; if a == b and c == d => det(M) =0
     // For an MDS matrix, every possible mxm submatrix, must have det(M) != 0
-    let xs: Vec<E::Fr> = (0..t as u64).map(scalar_from_u64).collect();
-    let ys: Vec<E::Fr> = (t as u64..2 * t as u64).map(scalar_from_u64).collect();
+    let xs: Vec<F> = (0..t as u64).map(F::from).collect();
+    let ys: Vec<F> = (t as u64..2 * t as u64).map(F::from).collect();
 
     let matrix = xs
         .iter()
@@ -138,32 +136,32 @@ fn generate_mds<E: ScalarEngine>(t: usize) -> Matrix<Scalar<E>> {
                 .map(|ys_item| {
                     // Generate the entry at (i,j)
                     let mut tmp = *xs_item;
-                    tmp.add_assign(&ys_item);
-                    tmp.inverse().unwrap()
+                    tmp.add_assign(ys_item);
+                    tmp.invert().unwrap()
                 })
                 .collect()
         })
         .collect();
 
     // To ensure correctness, we would check all sub-matrices for invertibility. Meanwhile, this is a simple sanity check.
-    assert!(is_invertible::<E>(&matrix));
+    assert!(is_invertible(&matrix));
 
     //  `poseidon::product_mds_with_matrix` relies on the constructed MDS matrix being symmetric, so ensure it is.
-    assert_eq!(matrix, transpose::<E>(&matrix));
+    assert_eq!(matrix, transpose(&matrix));
     matrix
 }
 
-fn make_prime<E: ScalarEngine>(m: &Matrix<Scalar<E>>) -> Matrix<Scalar<E>> {
+fn make_prime<F: PrimeField>(m: &Matrix<F>) -> Matrix<F> {
     m.iter()
         .enumerate()
         .map(|(i, row)| match i {
             0 => {
-                let mut new_row = vec![Scalar::<E>::zero(); row.len()];
-                new_row[0] = Scalar::<E>::one();
+                let mut new_row = vec![F::zero(); row.len()];
+                new_row[0] = F::one();
                 new_row
             }
             _ => {
-                let mut new_row = vec![Scalar::<E>::zero(); row.len()];
+                let mut new_row = vec![F::zero(); row.len()];
                 new_row[1..].copy_from_slice(&row[1..]);
                 new_row
             }
@@ -171,12 +169,9 @@ fn make_prime<E: ScalarEngine>(m: &Matrix<Scalar<E>>) -> Matrix<Scalar<E>> {
         .collect()
 }
 
-fn make_double_prime<E: ScalarEngine>(
-    m: &Matrix<Scalar<E>>,
-    m_hat_inv: &Matrix<Scalar<E>>,
-) -> Matrix<Scalar<E>> {
-    let (v, w) = make_v_w::<E>(m);
-    let w_hat = apply_matrix::<E>(m_hat_inv, &w);
+fn make_double_prime<F: PrimeField>(m: &Matrix<F>, m_hat_inv: &Matrix<F>) -> Matrix<F> {
+    let (v, w) = make_v_w(m);
+    let w_hat = apply_matrix(m_hat_inv, &w);
 
     m.iter()
         .enumerate()
@@ -188,16 +183,16 @@ fn make_double_prime<E: ScalarEngine>(
                 new_row
             }
             _ => {
-                let mut new_row = vec![Scalar::<E>::zero(); row.len()];
+                let mut new_row = vec![F::zero(); row.len()];
                 new_row[0] = w_hat[i - 1];
-                new_row[i] = Scalar::<E>::one();
+                new_row[i] = F::one();
                 new_row
             }
         })
         .collect()
 }
 
-fn make_v_w<E: ScalarEngine>(m: &Matrix<Scalar<E>>) -> (Vec<Scalar<E>>, Vec<Scalar<E>>) {
+fn make_v_w<F: PrimeField>(m: &Matrix<F>) -> (Vec<F>, Vec<F>) {
     let v = m[0][1..].to_vec();
     let w = m.iter().skip(1).map(|column| column[0]).collect();
     (v, w)
@@ -207,7 +202,8 @@ fn make_v_w<E: ScalarEngine>(m: &Matrix<Scalar<E>>) -> (Vec<Scalar<E>>, Vec<Scal
 mod tests {
     use super::*;
     use crate::*;
-    use bellperson::bls::{Bls12, Fr};
+    use blstrs::Scalar as Fr;
+    use ff::Field;
     use matrix::left_apply_matrix;
     use rand::SeedableRng;
     use rand_xorshift::XorShiftRng;
@@ -227,7 +223,7 @@ mod tests {
             m_hat_inv: _,
             m_prime,
             m_double_prime,
-        } = create_mds_matrices::<Bls12>(width);
+        } = create_mds_matrices::<Fr>(width);
 
         for i in 0..m_hat.len() {
             for j in 0..m_hat[i].len() {
@@ -236,15 +232,10 @@ mod tests {
         }
 
         // M^-1 x M = I
-        assert!(matrix::is_identity::<Bls12>(
-            &matrix::mat_mul::<Bls12>(&m_inv, &m).unwrap()
-        ));
+        assert!(matrix::is_identity(&matrix::mat_mul(&m_inv, &m).unwrap()));
 
         // M' x M'' = M
-        assert_eq!(
-            m,
-            matrix::mat_mul::<Bls12>(&m_prime, &m_double_prime).unwrap()
-        );
+        assert_eq!(m, matrix::mat_mul(&m_prime, &m_double_prime).unwrap());
     }
 
     #[test]
@@ -262,7 +253,7 @@ mod tests {
             m_hat_inv: _,
             m_prime,
             m_double_prime,
-        } = create_mds_matrices::<Bls12>(width);
+        } = create_mds_matrices::<Fr>(width);
 
         let mut base = Vec::with_capacity(width);
         for _ in 0..width {
@@ -275,19 +266,18 @@ mod tests {
         let mut y = base;
         y[0] = Fr::random(&mut rng);
 
-        let qx = apply_matrix::<Bls12>(&m_prime, &x);
-        let qy = apply_matrix::<Bls12>(&m_prime, &y);
+        let qx = apply_matrix(&m_prime, &x);
+        let qy = apply_matrix(&m_prime, &y);
         assert_eq!(qx[0], x[0]);
         assert_eq!(qy[0], y[0]);
         assert_eq!(qx[1..], qy[1..]);
 
-        let mx = left_apply_matrix::<Bls12>(&m, &x);
-        let m1_m2_x =
-            left_apply_matrix::<Bls12>(&m_prime, &left_apply_matrix::<Bls12>(&m_double_prime, &x));
+        let mx = left_apply_matrix(&m, &x);
+        let m1_m2_x = left_apply_matrix(&m_prime, &left_apply_matrix(&m_double_prime, &x));
         assert_eq!(mx, m1_m2_x);
 
-        let xm = apply_matrix::<Bls12>(&m, &x);
-        let x_m1_m2 = apply_matrix::<Bls12>(&m_double_prime, &apply_matrix::<Bls12>(&m_prime, &x));
+        let xm = apply_matrix(&m, &x);
+        let x_m1_m2 = apply_matrix(&m_double_prime, &apply_matrix(&m_prime, &x));
         assert_eq!(xm, x_m1_m2);
 
         let mut rk = Vec::with_capacity(width);
@@ -306,10 +296,10 @@ mod tests {
     fn test_factor_to_sparse_matrices_aux(width: usize, n: usize) {
         let mut rng = XorShiftRng::from_seed(crate::TEST_SEED);
 
-        let m = generate_mds::<Bls12>(width);
+        let m = generate_mds::<Fr>(width);
         let m2 = m.clone();
 
-        let (pre_sparse, mut sparse) = factor_to_sparse_matrices::<Bls12>(m, n);
+        let (pre_sparse, mut sparse) = factor_to_sparse_matrices(m, n);
         assert_eq!(n, sparse.len());
 
         let mut initial = Vec::with_capacity(width);
@@ -325,8 +315,8 @@ mod tests {
         let expected = std::iter::repeat(m2).take(n + 1).zip(&round_keys).fold(
             initial.clone(),
             |mut acc, (m, rk)| {
-                acc = apply_matrix::<Bls12>(&m, &acc);
-                quintic_s_box::<Bls12>(&mut acc[0], None, Some(&rk));
+                acc = apply_matrix(&m, &acc);
+                quintic_s_box(&mut acc[0], None, Some(&rk));
                 acc
             },
         );
@@ -336,8 +326,8 @@ mod tests {
             .iter()
             .zip(&round_keys)
             .fold(initial, |mut acc, (m, rk)| {
-                acc = apply_matrix::<Bls12>(&m, &acc);
-                quintic_s_box::<Bls12>(&mut acc[0], None, Some(&rk));
+                acc = apply_matrix(&m, &acc);
+                quintic_s_box(&mut acc[0], None, Some(&rk));
                 acc
             });
         assert_eq!(expected, actual);
@@ -351,13 +341,13 @@ mod tests {
     }
 
     fn test_factor_to_sparse_matrixes_aux(width: usize, n: usize) {
-        let m = generate_mds::<Bls12>(width);
+        let m = generate_mds::<Fr>(width);
         let m2 = m.clone();
 
-        let (pre_sparse, sparse_matrices) = factor_to_sparse_matrices::<Bls12>(m, n);
+        let (pre_sparse, sparse_matrices) = factor_to_sparse_matrices(m, n);
         assert_eq!(n, sparse_matrices.len());
 
-        let (pre_sparse2, sparse_matrixes) = factor_to_sparse_matrixes::<Bls12>(m2, n);
+        let (pre_sparse2, sparse_matrixes) = factor_to_sparse_matrixes(m2, n);
 
         assert_eq!(pre_sparse, pre_sparse2);
 
